@@ -19,8 +19,10 @@ import os
 import copy
 import re
 import collections
+import io
 import itertools
 import warnings
+import six
 from six.moves import urllib
 # Make defaultdict compatible with versions of python older than 2.4
 try:
@@ -384,20 +386,46 @@ class _AbstractMapReduceGFF:
                 else:
                     val = tuple(parts[1:])
                 # specific directives that need special handling
-                if key == "sequence-region": # convert to Python 0-based coordinates
-                    val = (val[0], int(val[1]) - 1, int(val[2]))
+                if key == "sequence-region":  # convert to Python 0-based coordinates
+                    if len(val) == 2:  # handle regions missing contig
+                        val = (int(val[0]) - 1, int(val[1]))
+                    elif len(val) == 3:
+                        val = (val[0], int(val[1]) - 1, int(val[2]))
                 dir_keyvals[key].append(val)
         for key, vals in dir_keyvals.items():
             for rec in base.values():
                 self._add_ann_to_rec(rec, key, vals)
         return base
 
+    def _get_matching_record_id(self, base, find_id):
+        """Find a matching base record with the test identifier, handling tricky cases.
+
+        NCBI IDs https://en.wikipedia.org/wiki/FASTA_format#NCBI_identifiers
+        """
+        # Straight matches for identifiers
+        if find_id in base:
+            return find_id
+        # NCBI style IDs in find_id
+        elif find_id and find_id.find("|") > 0:
+            for test_id in [x.strip() for x in find_id.split("|")[1:]]:
+                if test_id and test_id in base:
+                    return test_id
+        # NCBI style IDs in base IDs
+        else:
+            for base_id in base.keys():
+                if base_id.find("|") > 0:
+                    for test_id in [x.strip() for x in base_id.split("|")[1:]]:
+                        if test_id and test_id == find_id:
+                            return base_id
+        return None
+
     def _add_seqs(self, base, recs):
         """Add sequence information contained in the GFF3 to records.
         """
         for rec in recs:
-            if rec.id in base:
-                base[rec.id].seq = rec.seq
+            match_id = self._get_matching_record_id(base, rec.id)
+            if match_id:
+                base[match_id].seq = rec.seq
             else:
                 base[rec.id] = rec
         return base
@@ -504,19 +532,20 @@ class _AbstractMapReduceGFF:
         """Retrieve a record to add features to.
         """
         max_loc = info_dict.get('location', (0, 1))[1]
-        try:
-            cur_rec = base[info_dict['rec_id']]
+        match_id = self._get_matching_record_id(base, info_dict['rec_id'])
+        if match_id:
+            cur_rec = base[match_id]
             # update generated unknown sequences with the expected maximum length
             if isinstance(cur_rec.seq, UnknownSeq):
                 cur_rec.seq._length = max([max_loc, cur_rec.seq._length])
             return cur_rec, base
-        except KeyError:
-            if self._create_missing:
-                new_rec = SeqRecord(UnknownSeq(max_loc), info_dict['rec_id'])
-                base[info_dict['rec_id']] = new_rec
-                return new_rec, base
-            else:
-                raise
+        elif self._create_missing:
+            new_rec = SeqRecord(UnknownSeq(max_loc), info_dict['rec_id'])
+            base[info_dict['rec_id']] = new_rec
+            return new_rec, base
+        else:
+            raise KeyError("Did not find matching record in %s for %s" %
+                           (base.keys(), info_dict))
 
     def _add_missing_parent(self, base, parent_id, cur_children):
         """Add a new feature that is missing from the GFF file.
@@ -687,8 +716,14 @@ class GFFParser(_AbstractMapReduceGFF):
                 return self
             def __next__(self):
                 return next(self._iter)
-            def read(self):
-                return "".join(l for l in self._iter)
+            next = __next__
+            def read(self, size=-1):
+                if size < 0:
+                    return "".join(l for l in self._iter)
+                elif size == 0:
+                    return ""  # Used by Biopython to sniff unicode vs bytes
+                else:
+                    raise NotImplementedError
             def readline(self):
                 try:
                     return next(self._iter)
@@ -756,6 +791,8 @@ def parse_simple(gff_files, limit_info=None):
             yield rec["child"][0]
         elif "parent" in rec:
             yield rec["parent"][0]
+        elif "feature" in rec:
+            yield rec["feature"][0]
         # ignore directive lines
         else:
             assert "directive" in rec
@@ -768,6 +805,8 @@ def _file_or_handle(fn):
         if hasattr(in_file, "read"):
             need_close = False
             in_handle = in_file
+            if six.PY3 and not isinstance(in_handle, io.TextIOBase):
+                raise TypeError('input handle must be opened in text mode')
         else:
             need_close = True
             in_handle = open(in_file)
